@@ -25,9 +25,7 @@
 
 #include <signal.h>
 #include <string.h>
-#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
-#endif
 #include <stdlib.h>
 
 #include <gst/interfaces/xoverlay.h>
@@ -50,6 +48,15 @@ GST_DEBUG_CATEGORY_EXTERN (sdl_debug);
 
 #define I420_SIZE(w,h)     (I420_V_OFFSET(w,h)+(I420_V_ROWSTRIDE(w)*GST_ROUND_UP_2(h)/2))
 
+/* elementfactory information */
+static const GstElementDetails gst_sdlvideosink_details =
+GST_ELEMENT_DETAILS ("SDL video sink",
+    "Sink/Video",
+    "An SDL-based videosink",
+    "Ronald Bultje <rbultje@ronald.bitfreak.net>"
+    "Edgard Lima <edgard.lima@indt.org.br>"
+    "Jan Schmidt <thaytan@mad.scientist.com>");
+
 
 enum
 {
@@ -63,8 +70,8 @@ static gboolean gst_sdlvideosink_supported (GstImplementsInterface * iface,
     GType type);
 
 static void gst_sdlvideosink_xoverlay_init (GstXOverlayClass * klass);
-static void gst_sdlvideosink_xoverlay_set_window_handle
-    (GstXOverlay * overlay, guintptr parent);
+static void gst_sdlvideosink_xoverlay_set_xwindow_id
+    (GstXOverlay * overlay, unsigned long parent);
 
 static gboolean gst_sdlvideosink_lock (GstSDLVideoSink * sdl);
 static void gst_sdlvideosink_unlock (GstSDLVideoSink * sdl);
@@ -131,9 +138,11 @@ gst_sdlvideosink_base_init (gpointer g_class)
   guint32 formats[] = {
     GST_MAKE_FOURCC ('I', '4', '2', '0'),
     GST_MAKE_FOURCC ('Y', 'V', '1', '2'),
-    GST_MAKE_FOURCC ('Y', 'U', 'Y', '2'),
-    GST_MAKE_FOURCC ('Y', 'V', 'Y', 'U'),
-    GST_MAKE_FOURCC ('U', 'Y', 'V', 'Y')
+    GST_MAKE_FOURCC ('Y', 'U', 'Y', '2')
+        /*
+           GST_MAKE_FOURCC ('Y', 'V', 'Y', 'U'),
+           GST_MAKE_FOURCC ('U', 'Y', 'V', 'Y')
+         */
   };
 
   /* make a list of all available caps */
@@ -151,12 +160,8 @@ gst_sdlvideosink_base_init (gpointer g_class)
       GST_PAD_SINK, GST_PAD_ALWAYS, capslist);
 
   gst_element_class_add_pad_template (element_class, sink_template);
-  gst_object_unref (sink_template);
-  gst_element_class_set_details_simple (element_class, "SDL video sink",
-      "Sink/Video", "An SDL-based videosink",
-      "Ronald Bultje <rbultje@ronald.bitfreak.net>, "
-      "Edgard Lima <edgard.lima@indt.org.br>, "
-      "Jan Schmidt <thaytan@mad.scientist.com>");
+  gst_element_class_set_details (element_class, &gst_sdlvideosink_details);
+
 }
 
 static void
@@ -218,8 +223,7 @@ gst_sdlvideosink_class_init (GstSDLVideoSinkClass * klass)
 
   g_object_class_install_property (gobject_class, PROP_FULLSCREEN,
       g_param_spec_boolean ("fullscreen", "Fullscreen",
-          "If true it will be Full screen", FALSE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          "If true it will be Full screen", FALSE, G_PARAM_READWRITE));
 
   /*gstvs_class->set_video_out = gst_sdlvideosink_set_video_out;
      gstvs_class->push_ui_event = gst_sdlvideosink_push_ui_event;
@@ -363,15 +367,14 @@ gst_sdlvideosink_supported (GstImplementsInterface * interface,
 static void
 gst_sdlvideosink_xoverlay_init (GstXOverlayClass * klass)
 {
-  klass->set_window_handle = gst_sdlvideosink_xoverlay_set_window_handle;
+  klass->set_xwindow_id = gst_sdlvideosink_xoverlay_set_xwindow_id;
 }
 
 static void
-gst_sdlvideosink_xoverlay_set_window_handle (GstXOverlay * overlay,
-    guintptr handle)
+gst_sdlvideosink_xoverlay_set_xwindow_id (GstXOverlay * overlay,
+    unsigned long parent)
 {
   GstSDLVideoSink *sdlvideosink = GST_SDLVIDEOSINK (overlay);
-  unsigned long parent = (unsigned long) handle;
 
   if (sdlvideosink->xwindow_id == parent)
     return;
@@ -493,7 +496,7 @@ gst_sdlv_process_events (GstSDLVideoSink * sdlvideosink)
 {
   SDL_Event event;
   int numevents;
-  char *keysym = NULL;
+  char *keysym;
 
   do {
     SDL_PumpEvents ();
@@ -590,12 +593,12 @@ gst_sdlvideosink_initsdl (GstSDLVideoSink * sdlvideosink)
   }
 
   if (!sdlvideosink->xwindow_id) {
-    g_unsetenv ("SDL_WINDOWID");
+    unsetenv ("SDL_WINDOWID");
   } else {
     char SDL_hack[32];
 
     sprintf (SDL_hack, "%lu", sdlvideosink->xwindow_id);
-    g_setenv ("SDL_WINDOWID", SDL_hack, 1);
+    setenv ("SDL_WINDOWID", SDL_hack, 1);
   }
 
   /* Initialize the SDL library */
@@ -714,6 +717,7 @@ static gboolean
 gst_sdlvideosink_setcaps (GstBaseSink * bsink, GstCaps * vscapslist)
 {
   GstSDLVideoSink *sdlvideosink;
+  guint32 format;
   GstStructure *structure;
   gboolean res = TRUE;
 
@@ -742,6 +746,7 @@ gst_sdlvideosink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
 {
 
   GstSDLVideoSink *sdlvideosink;
+  SDL_Event sdl_event;
 
   sdlvideosink = GST_SDLVIDEOSINK (bsink);
 
@@ -775,9 +780,7 @@ gst_sdlvideosink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
           v = y + I420_V_OFFSET (sdlvideosink->width, sdlvideosink->height);
           break;
         default:
-          gst_sdlvideosink_unlock (sdlvideosink);
-          g_mutex_unlock (sdlvideosink->lock);
-          g_return_val_if_reached (GST_FLOW_ERROR);
+          g_assert_not_reached ();
       }
 
       /* Y Plane */
@@ -945,7 +948,8 @@ gst_sdlvideosink_navigation_send_event (GstNavigation * navigation,
   GstSDLVideoSink *sdlvideosink = GST_SDLVIDEOSINK (navigation);
   GstEvent *event;
   GstVideoRectangle src, dst, result;
-  double x, y, old_x, old_y;
+  gint width, height;
+  double x, y;
   GstPad *pad = NULL;
 
   src.w = GST_VIDEO_SINK_WIDTH (sdlvideosink);
@@ -959,8 +963,8 @@ gst_sdlvideosink_navigation_send_event (GstNavigation * navigation,
   /* Our coordinates can be wrong here if we centered the video */
 
   /* Converting pointer coordinates to the non scaled geometry */
-  if (gst_structure_get_double (structure, "pointer_x", &old_x)) {
-    x = old_x;
+  if (gst_structure_get_double (structure, "pointer_x", &x)) {
+    double old_x = x;
 
     if (x >= result.x && x <= (result.x + result.w)) {
       x -= result.x;
@@ -973,8 +977,8 @@ gst_sdlvideosink_navigation_send_event (GstNavigation * navigation,
         "coordinate from %f to %f", old_x, x);
     gst_structure_set (structure, "pointer_x", G_TYPE_DOUBLE, x, NULL);
   }
-  if (gst_structure_get_double (structure, "pointer_y", &old_y)) {
-    y = old_y;
+  if (gst_structure_get_double (structure, "pointer_y", &y)) {
+    double old_y = y;
 
     if (y >= result.y && y <= (result.y + result.h)) {
       y -= result.y;
