@@ -36,6 +36,13 @@
 #include <gst/video/video.h>
 #include "gstxvidenc.h"
 
+/* elementfactory information */
+static const GstElementDetails gst_xvidenc_details =
+GST_ELEMENT_DETAILS ("XviD video encoder",
+    "Codec/Encoder/Video",
+    "XviD encoder based on xvidcore",
+    "Ronald Bultje <rbultje@ronald.bitfreak.net>");
+
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -51,41 +58,18 @@ static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
 static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("video/mpeg, "
-        "mpegversion = (int) 4, "
-        "systemstream = (boolean) FALSE, "
+    GST_STATIC_CAPS ("video/x-xvid, "
         "width = (int) [ 0, MAX ], "
-        "height = (int) [ 0, MAX ], "
-        "framerate = (fraction) [ 0/1, MAX ], "
-        "profile = (string) simple, "
-        "level = (string) { 0, 1, 2, 3, 4a, 5, 6 };"
-        "video/mpeg, "
-        "mpegversion = (int) 4, "
-        "systemstream = (boolean) FALSE, "
-        "width = (int) [ 0, MAX ], "
-        "height = (int) [ 0, MAX ], "
-        "framerate = (fraction) [ 0/1, MAX ], "
-        "profile = (string) advanced-real-time-simple, "
-        "level = (string) { 1, 2, 3, 4 };"
-        "video/mpeg, "
-        "mpegversion = (int) 4, "
-        "systemstream = (boolean) FALSE, "
-        "width = (int) [ 0, MAX ], "
-        "height = (int) [ 0, MAX ], "
-        "framerate = (fraction) [ 0/1, MAX ], "
-        "profile = (string) advanced-simple, "
-        "level = (string) { 0, 1, 2, 3, 4 };"
-        "video/mpeg, "
-        "mpegversion = (int) 4, "
-        "systemstream = (boolean) FALSE, "
-        "width = (int) [ 0, MAX ], " "height = (int) [ 0, MAX ]; "
-        "video/x-xvid, "
-        "width = (int) [ 0, MAX ], "
-        "height = (int) [ 0, MAX ], " "framerate = (fraction) [ 0/1, MAX ];")
+        "height = (int) [ 0, MAX ], " "framerate = (fraction) [ 0/1, MAX ]")
     );
 
 
-/* XvidEnc properties */
+/* XvidEnc signals and properties */
+enum
+{
+  FRAME_ENCODED,
+  LAST_SIGNAL
+};
 
 /* maximum property-id */
 static int xvidenc_prop_count;
@@ -102,7 +86,6 @@ static void gst_xvidenc_init (GstXvidEnc * xvidenc);
 static void gst_xvidenc_finalize (GObject * object);
 static GstFlowReturn gst_xvidenc_chain (GstPad * pad, GstBuffer * data);
 static gboolean gst_xvidenc_setcaps (GstPad * pad, GstCaps * vscapslist);
-static GstCaps *gst_xvidenc_getcaps (GstPad * pad);
 static void gst_xvidenc_flush_buffers (GstXvidEnc * xvidenc, gboolean send);
 static gboolean gst_xvidenc_handle_sink_event (GstPad * pad, GstEvent * event);
 
@@ -115,6 +98,7 @@ static GstStateChangeReturn gst_xvidenc_change_state (GstElement * element,
     GstStateChange transition);
 
 static GstElementClass *parent_class = NULL;
+static guint gst_xvidenc_signals[LAST_SIGNAL] = { 0 };
 
 #define GST_TYPE_XVIDENC_PROFILE (gst_xvidenc_profile_get_type ())
 static GType
@@ -129,9 +113,6 @@ gst_xvidenc_profile_get_type (void)
       {XVID_PROFILE_S_L1, "S_L1", "Simple profile, L1"},
       {XVID_PROFILE_S_L2, "S_L2", "Simple profile, L2"},
       {XVID_PROFILE_S_L3, "S_L3", "Simple profile, L3"},
-      {XVID_PROFILE_S_L4a, "S_L4a", "Simple profile, L4a"},
-      {XVID_PROFILE_S_L5, "S_L5", "Simple profile, L5"},
-      {XVID_PROFILE_S_L6, "S_L6", "Simple profile, L6"},
       {XVID_PROFILE_ARTS_L1, "ARTS_L1",
           "Advanced real-time simple profile, L1"},
       {XVID_PROFILE_ARTS_L2, "ARTS_L2",
@@ -224,17 +205,9 @@ gst_xvidenc_get_type (void)
       0,
       (GInstanceInitFunc) gst_xvidenc_init,
     };
-    const GInterfaceInfo preset_interface_info = {
-      NULL,                     /* interface_init */
-      NULL,                     /* interface_finalize */
-      NULL                      /* interface_data */
-    };
 
     xvidenc_type = g_type_register_static (GST_TYPE_ELEMENT,
         "GstXvidEnc", &xvidenc_info, 0);
-
-    g_type_add_interface_static (xvidenc_type, GST_TYPE_PRESET,
-        &preset_interface_info);
   }
   return xvidenc_type;
 }
@@ -244,12 +217,11 @@ gst_xvidenc_base_init (GstXvidEncClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class, &sink_template);
-  gst_element_class_add_static_pad_template (element_class, &src_template);
-  gst_element_class_set_details_simple (element_class, "XviD video encoder",
-      "Codec/Encoder/Video",
-      "XviD encoder based on xvidcore",
-      "Ronald Bultje <rbultje@ronald.bitfreak.net>");
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_template));
+  gst_element_class_set_details (element_class, &gst_xvidenc_details);
 }
 
 /* add property pspec to klass using the counter count,
@@ -298,212 +270,200 @@ gst_xvidenc_class_init (GstXvidEncClass * klass)
 
   pspec = g_param_spec_enum ("profile", "Profile",
       "XviD/MPEG-4 encoding profile",
-      GST_TYPE_XVIDENC_PROFILE, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      GST_TYPE_XVIDENC_PROFILE, 0, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, profile);
 
   pspec = g_param_spec_enum ("quant-type", "Quantizer Type",
-      "Quantizer type", GST_TYPE_XVIDENC_QUANT_TYPE, 0,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      "Quantizer type", GST_TYPE_XVIDENC_QUANT_TYPE, 0, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, quant_type);
 
   pspec = g_param_spec_enum ("pass", "Encoding pass/type",
       "Encoding pass/type",
-      GST_TYPE_XVIDENC_PASS, XVIDENC_CBR,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      GST_TYPE_XVIDENC_PASS, XVIDENC_CBR, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, pass);
 
   pspec = g_param_spec_int ("bitrate", "Bitrate",
       "[CBR|PASS2] Target video bitrate (bps)",
-      0, G_MAXINT, 1800000, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      0, G_MAXINT, 1800000, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, bitrate);
 
   pspec = g_param_spec_int ("quantizer", "Quantizer",
       "[QUANT] Quantizer to apply for constant quantizer mode",
-      2, 31, 2, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      2, 31, 2, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, quant);
 
   pspec = g_param_spec_string ("statsfile", "Statistics Filename",
       "[PASS1|PASS2] Filename to store data for 2-pass encoding",
-      "xvid-stats.log", G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      "xvid-stats.log", G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, filename);
 
   pspec = g_param_spec_int ("max-key-interval", "Max. Key Interval",
       "Maximum number of frames between two keyframes (< 0 is in sec)",
-      -100, G_MAXINT, -10, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      -100, G_MAXINT, -10, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, max_key_interval);
 
   pspec = g_param_spec_boolean ("closed-gop", "Closed GOP",
-      "Closed GOP", FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      "Closed GOP", FALSE, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, closed_gop);
 
   pspec = g_param_spec_int ("motion", "ME Quality",
-      "Quality of Motion Estimation", 0, 6, 6,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      "Quality of Motion Estimation", 0, 6, 6, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, motion);
 
   pspec = g_param_spec_boolean ("me-chroma", "ME Chroma",
       "Enable use of Chroma planes for Motion Estimation",
-      TRUE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      TRUE, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, me_chroma);
 
   pspec = g_param_spec_int ("me-vhq", "ME DCT/Frequency",
       "Extent in which to use DCT to minimize encoding length",
-      0, 4, 1, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      0, 4, 1, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, me_vhq);
 
   pspec = g_param_spec_boolean ("me-quarterpel", "ME Quarterpel",
       "Use quarter pixel precision for motion vector search",
-      FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      FALSE, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, me_quarterpel);
 
   pspec = g_param_spec_boolean ("lumimasking", "Lumimasking",
       "Enable lumimasking - apply more compression to dark or bright areas",
-      FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      FALSE, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, lumimasking);
 
   pspec = g_param_spec_int ("max-bframes", "Max B-Frames",
-      "Maximum B-frames in a row", 0, G_MAXINT, 1,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      "Maximum B-frames in a row", 0, G_MAXINT, 1, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, max_bframes);
 
   pspec = g_param_spec_int ("bquant-ratio", "B-quantizer ratio",
-      "Ratio in B-frame quantizer computation", 0, 200, 150,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      "Ratio in B-frame quantizer computation", 0, 200, 150, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, bquant_ratio);
 
   pspec = g_param_spec_int ("bquant-offset", "B-quantizer offset",
       "Offset in B-frame quantizer computation",
-      0, 200, 100, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      0, 200, 100, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, bquant_offset);
 
   pspec = g_param_spec_int ("bframe-threshold", "B-Frame Threshold",
       "Higher threshold yields more chance that B-frame is used",
-      -255, 255, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      -255, 255, 0, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, bframe_threshold);
 
   pspec = g_param_spec_boolean ("gmc", "Global Motion Compensation",
       "Allow generation of Sprite Frames for Pan/Zoom/Rotating images",
-      FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      FALSE, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, gmc);
 
   pspec = g_param_spec_boolean ("trellis", "Trellis Quantization",
-      "Enable Trellis Quantization", FALSE,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      "Enable Trellis Quantization", FALSE, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, trellis);
 
   pspec = g_param_spec_boolean ("interlaced", "Interlaced Material",
-      "Enable for interlaced video material", FALSE,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      "Enable for interlaced video material", FALSE, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, interlaced);
 
   pspec = g_param_spec_boolean ("cartoon", "Cartoon Material",
-      "Adjust thresholds for flat looking cartoons", FALSE,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      "Adjust thresholds for flat looking cartoons", FALSE, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, cartoon);
 
   pspec = g_param_spec_boolean ("greyscale", "Disable Chroma",
-      "Do not write chroma data in encoded video", FALSE,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      "Do not write chroma data in encoded video", FALSE, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, greyscale);
 
   pspec = g_param_spec_boolean ("hqacpred", "High quality AC prediction",
-      "Enable high quality AC prediction", TRUE,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      "Enable high quality AC prediction", TRUE, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, hqacpred);
 
   pspec = g_param_spec_int ("max-iquant", "Max Quant I-Frames",
-      "Upper bound for I-frame quantization", 0, 31, 31,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      "Upper bound for I-frame quantization", 0, 31, 31, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, max_iquant);
 
   pspec = g_param_spec_int ("min-iquant", "Min Quant I-Frames",
-      "Lower bound for I-frame quantization", 0, 31, 2,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      "Lower bound for I-frame quantization", 0, 31, 2, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, min_iquant);
 
   pspec = g_param_spec_int ("max-pquant", "Max Quant P-Frames",
-      "Upper bound for P-frame quantization", 0, 31, 31,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      "Upper bound for P-frame quantization", 0, 31, 31, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, max_pquant);
 
   pspec = g_param_spec_int ("min-pquant", "Min Quant P-Frames",
-      "Lower bound for P-frame quantization", 0, 31, 2,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      "Lower bound for P-frame quantization", 0, 31, 2, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, min_pquant);
 
   pspec = g_param_spec_int ("max-bquant", "Max Quant B-Frames",
-      "Upper bound for B-frame quantization", 0, 31, 31,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      "Upper bound for B-frame quantization", 0, 31, 31, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, max_bquant);
 
   pspec = g_param_spec_int ("min-bquant", "Min Quant B-Frames",
-      "Lower bound for B-frame quantization", 0, 31, 2,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      "Lower bound for B-frame quantization", 0, 31, 2, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, min_bquant);
 
   pspec = g_param_spec_int ("reaction-delay-factor", "Reaction Delay Factor",
-      "[CBR] Reaction delay factor", -1, 100, -1,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      "[CBR] Reaction delay factor", -1, 100, -1, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, reaction_delay_factor);
 
   pspec = g_param_spec_int ("averaging-period", "Averaging Period",
       "[CBR] Number of frames for which XviD averages bitrate",
-      -1, 100, -1, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      -1, 100, -1, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, averaging_period);
 
   pspec = g_param_spec_int ("buffer", "Buffer Size",
-      "[CBR] Size of the video buffers", -1, G_MAXINT, -1,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      "[CBR] Size of the video buffers", -1, G_MAXINT, -1, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, buffer);
 
   pspec = g_param_spec_int ("keyframe-boost", "Keyframe boost",
-      "[PASS2] Bitrate boost for keyframes", 0, 100, 0,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      "[PASS2] Bitrate boost for keyframes", 0, 100, 0, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, keyframe_boost);
 
   pspec = g_param_spec_int ("curve-compression-high", "Curve Compression High",
       "[PASS2] Shrink factor for upper part of bitrate curve",
-      0, 100, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      0, 100, 0, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, curve_compression_high);
 
   pspec = g_param_spec_int ("curve-compression-low", "Curve Compression Low",
       "[PASS2] Growing factor for lower part of bitrate curve",
-      0, 100, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      0, 100, 0, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, curve_compression_low);
 
   pspec = g_param_spec_int ("flow-control-strength", "Flow Control Strength",
       "[PASS2] Overflow control strength per frame",
-      -1, 100, 5, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      -1, 100, 5, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, overflow_control_strength);
 
   pspec =
       g_param_spec_int ("max-overflow-improvement", "Max Overflow Improvement",
       "[PASS2] Amount in % that flow control can increase frame size compared to ideal curve",
-      -1, 100, 5, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      -1, 100, 5, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, max_overflow_improvement);
 
   pspec =
       g_param_spec_int ("max-overflow-degradation", "Max Overflow Degradation",
       "[PASS2] Amount in % that flow control can decrease frame size compared to ideal curve",
-      -1, 100, 5, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      -1, 100, 5, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, max_overflow_degradation);
 
   pspec = g_param_spec_int ("keyframe-reduction", "Keyframe Reduction",
       "[PASS2] Keyframe size reduction in % of those within threshold",
-      -1, 100, 20, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      -1, 100, 20, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, kfreduction);
 
   pspec = g_param_spec_int ("keyframe-threshold", "Keyframe Threshold",
       "[PASS2] Distance between keyframes not to be subject to reduction",
-      -1, 100, 1, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      -1, 100, 1, G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, kfthreshold);
 
   pspec =
       g_param_spec_int ("container-frame-overhead", "Container Frame Overhead",
       "[PASS2] Average container overhead per frame", -1, 100, -1,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, container_frame_overhead);
 
+  /* signals */
+  gst_xvidenc_signals[FRAME_ENCODED] =
+      g_signal_new ("frame-encoded", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST,
+      G_STRUCT_OFFSET (GstXvidEncClass, frame_encoded),
+      NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
+
   gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_xvidenc_change_state);
 }
 
@@ -515,20 +475,22 @@ gst_xvidenc_init (GstXvidEnc * xvidenc)
   guint i, num_props;
 
   /* create the sink pad */
-  xvidenc->sinkpad = gst_pad_new_from_static_template (&sink_template, "sink");
+  xvidenc->sinkpad =
+      gst_pad_new_from_template (gst_static_pad_template_get (&sink_template),
+      "sink");
   gst_element_add_pad (GST_ELEMENT (xvidenc), xvidenc->sinkpad);
 
   gst_pad_set_chain_function (xvidenc->sinkpad,
       GST_DEBUG_FUNCPTR (gst_xvidenc_chain));
   gst_pad_set_setcaps_function (xvidenc->sinkpad,
       GST_DEBUG_FUNCPTR (gst_xvidenc_setcaps));
-  gst_pad_set_getcaps_function (xvidenc->sinkpad,
-      GST_DEBUG_FUNCPTR (gst_xvidenc_getcaps));
   gst_pad_set_event_function (xvidenc->sinkpad,
       GST_DEBUG_FUNCPTR (gst_xvidenc_handle_sink_event));
 
   /* create the src pad */
-  xvidenc->srcpad = gst_pad_new_from_static_template (&src_template, "src");
+  xvidenc->srcpad =
+      gst_pad_new_from_template (gst_static_pad_template_get (&src_template),
+      "src");
   gst_element_add_pad (GST_ELEMENT (xvidenc), xvidenc->srcpad);
   gst_pad_use_fixed_caps (xvidenc->srcpad);
 
@@ -551,7 +513,6 @@ gst_xvidenc_init (GstXvidEnc * xvidenc)
     g_param_value_set_default (pspec, &val);
     g_object_set_property (G_OBJECT (xvidenc), g_param_spec_get_name (pspec),
         &val);
-    g_value_unset (&val);
   }
 
   g_free (pspecs);
@@ -574,8 +535,6 @@ gst_xvidenc_finalize (GObject * object)
   GstXvidEnc *xvidenc = GST_XVIDENC (object);
 
   g_free (xvidenc->filename);
-
-  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static gboolean
@@ -602,97 +561,11 @@ gst_xvidenc_setup (GstXvidEnc * xvidenc)
   xvid_enc_create_t xenc;
   xvid_enc_plugin_t xplugin[2];
   gint ret;
-  GstCaps *allowed_caps;
-  gint profile = -1;
-
-  /* Negotiate profile/level with downstream */
-  allowed_caps = gst_pad_get_allowed_caps (xvidenc->srcpad);
-  if (allowed_caps && !gst_caps_is_empty (allowed_caps)) {
-    const gchar *profile_str, *level_str;
-
-    allowed_caps = gst_caps_make_writable (allowed_caps);
-    gst_caps_truncate (allowed_caps);
-
-    profile_str =
-        gst_structure_get_string (gst_caps_get_structure (allowed_caps, 0),
-        "profile");
-    level_str =
-        gst_structure_get_string (gst_caps_get_structure (allowed_caps, 0),
-        "level");
-    if (profile_str) {
-      if (g_str_equal (profile_str, "simple")) {
-        if (!level_str) {
-          profile = XVID_PROFILE_S_L0;
-        } else if (g_str_equal (level_str, "0")) {
-          profile = XVID_PROFILE_S_L0;
-        } else if (g_str_equal (level_str, "1")) {
-          profile = XVID_PROFILE_S_L1;
-        } else if (g_str_equal (level_str, "2")) {
-          profile = XVID_PROFILE_S_L2;
-        } else if (g_str_equal (level_str, "3")) {
-          profile = XVID_PROFILE_S_L3;
-        } else if (g_str_equal (level_str, "4a")) {
-          profile = XVID_PROFILE_S_L4a;
-        } else if (g_str_equal (level_str, "5")) {
-          profile = XVID_PROFILE_S_L5;
-        } else if (g_str_equal (level_str, "6")) {
-          profile = XVID_PROFILE_S_L6;
-        } else {
-          GST_ERROR_OBJECT (xvidenc,
-              "Invalid profile/level combination (%s %s)", profile_str,
-              level_str);
-        }
-      } else if (g_str_equal (profile_str, "advanced-real-time-simple")) {
-        if (!level_str) {
-          profile = XVID_PROFILE_ARTS_L1;
-        } else if (g_str_equal (level_str, "1")) {
-          profile = XVID_PROFILE_ARTS_L1;
-        } else if (g_str_equal (level_str, "2")) {
-          profile = XVID_PROFILE_ARTS_L2;
-        } else if (g_str_equal (level_str, "3")) {
-          profile = XVID_PROFILE_ARTS_L3;
-        } else if (g_str_equal (level_str, "4")) {
-          profile = XVID_PROFILE_ARTS_L4;
-        } else {
-          GST_ERROR_OBJECT (xvidenc,
-              "Invalid profile/level combination (%s %s)", profile_str,
-              level_str);
-        }
-      } else if (g_str_equal (profile_str, "advanced-simple")) {
-        if (!level_str) {
-          profile = XVID_PROFILE_AS_L0;
-        } else if (g_str_equal (level_str, "0")) {
-          profile = XVID_PROFILE_AS_L0;
-        } else if (g_str_equal (level_str, "1")) {
-          profile = XVID_PROFILE_AS_L1;
-        } else if (g_str_equal (level_str, "2")) {
-          profile = XVID_PROFILE_AS_L2;
-        } else if (g_str_equal (level_str, "3")) {
-          profile = XVID_PROFILE_AS_L3;
-        } else if (g_str_equal (level_str, "4")) {
-          profile = XVID_PROFILE_AS_L4;
-        } else {
-          GST_ERROR_OBJECT (xvidenc,
-              "Invalid profile/level combination (%s %s)", profile_str,
-              level_str);
-        }
-      } else {
-        GST_ERROR_OBJECT (xvidenc, "Invalid profile (%s)", profile_str);
-      }
-    }
-  }
-  if (allowed_caps)
-    gst_caps_unref (allowed_caps);
-
-  if (profile != -1) {
-    xvidenc->profile = profile;
-    g_object_notify (G_OBJECT (xvidenc), "profile");
-  }
 
   /* see xvid.h for the meaning of all this. */
   gst_xvid_init_struct (xenc);
 
-  xenc.profile = xvidenc->used_profile = xvidenc->profile;
+  xenc.profile = xvidenc->profile;
   xenc.width = xvidenc->width;
   xenc.height = xvidenc->height;
   xenc.max_bframes = xvidenc->max_bframes;
@@ -827,17 +700,11 @@ gst_xvidenc_setcaps (GstPad * pad, GstCaps * vscaps)
 
   structure = gst_caps_get_structure (vscaps, 0);
 
-  if (!gst_structure_get_int (structure, "width", &w) ||
-      !gst_structure_get_int (structure, "height", &h)) {
-    return FALSE;
-  }
-
+  g_return_val_if_fail (gst_structure_get_int (structure, "width", &w), FALSE);
+  g_return_val_if_fail (gst_structure_get_int (structure, "height", &h), FALSE);
   fps = gst_structure_get_value (structure, "framerate");
-  if (fps == NULL || !GST_VALUE_HOLDS_FRACTION (fps)) {
-    GST_WARNING_OBJECT (pad, "no framerate specified, or not a GstFraction");
-    return FALSE;
-  }
-
+  g_return_val_if_fail (w > 0 && h > 0
+      && fps != NULL && GST_VALUE_HOLDS_FRACTION (fps), FALSE);
   /* optional par info */
   par = gst_structure_get_value (structure, "pixel-aspect-ratio");
 
@@ -870,164 +737,37 @@ gst_xvidenc_setcaps (GstPad * pad, GstCaps * vscaps)
   xvidenc->xframe_cache = NULL;
 
   if (gst_xvidenc_setup (xvidenc)) {
-    gboolean ret = FALSE;
-    GstCaps *new_caps = NULL, *allowed_caps;
+    GstPadLinkReturn ret;
+    GstCaps *new_caps;
+    GstPad *peer;
 
-    /* please downstream with preferred caps */
-    allowed_caps = gst_pad_get_allowed_caps (xvidenc->srcpad);
-    GST_DEBUG_OBJECT (xvidenc, "allowed caps: %" GST_PTR_FORMAT, allowed_caps);
-
-    if (allowed_caps && !gst_caps_is_empty (allowed_caps)) {
-      new_caps = gst_caps_copy_nth (allowed_caps, 0);
-    } else {
-      new_caps = gst_caps_new_simple ("video/x-xvid", NULL);
-    }
-    if (allowed_caps)
-      gst_caps_unref (allowed_caps);
-
-    gst_caps_set_simple (new_caps,
+    new_caps = gst_caps_new_simple ("video/x-xvid",
         "width", G_TYPE_INT, w, "height", G_TYPE_INT, h,
         "framerate", GST_TYPE_FRACTION, xvidenc->fbase, xvidenc->fincr,
         "pixel-aspect-ratio", GST_TYPE_FRACTION,
         xvidenc->par_width, xvidenc->par_height, NULL);
-    /* just to be sure */
-    gst_pad_fixate_caps (xvidenc->srcpad, new_caps);
-
-    if (xvidenc->used_profile != 0) {
-      switch (xvidenc->used_profile) {
-        case XVID_PROFILE_S_L0:
-          gst_caps_set_simple (new_caps, "profile", G_TYPE_STRING, "simple",
-              "level", G_TYPE_STRING, "0", NULL);
-          break;
-        case XVID_PROFILE_S_L1:
-          gst_caps_set_simple (new_caps, "profile", G_TYPE_STRING, "simple",
-              "level", G_TYPE_STRING, "1", NULL);
-          break;
-        case XVID_PROFILE_S_L2:
-          gst_caps_set_simple (new_caps, "profile", G_TYPE_STRING, "simple",
-              "level", G_TYPE_STRING, "2", NULL);
-          break;
-        case XVID_PROFILE_S_L3:
-          gst_caps_set_simple (new_caps, "profile", G_TYPE_STRING, "simple",
-              "level", G_TYPE_STRING, "3", NULL);
-          break;
-        case XVID_PROFILE_S_L4a:
-          gst_caps_set_simple (new_caps, "profile", G_TYPE_STRING, "simple",
-              "level", G_TYPE_STRING, "4a", NULL);
-          break;
-        case XVID_PROFILE_S_L5:
-          gst_caps_set_simple (new_caps, "profile", G_TYPE_STRING, "simple",
-              "level", G_TYPE_STRING, "5", NULL);
-          break;
-        case XVID_PROFILE_S_L6:
-          gst_caps_set_simple (new_caps, "profile", G_TYPE_STRING, "simple",
-              "level", G_TYPE_STRING, "6", NULL);
-          break;
-        case XVID_PROFILE_ARTS_L1:
-          gst_caps_set_simple (new_caps, "profile", G_TYPE_STRING,
-              "advanced-real-time-simple", "level", G_TYPE_STRING, "1", NULL);
-          break;
-        case XVID_PROFILE_ARTS_L2:
-          gst_caps_set_simple (new_caps, "profile", G_TYPE_STRING,
-              "advanced-real-time-simple", "level", G_TYPE_STRING, "2", NULL);
-          break;
-        case XVID_PROFILE_ARTS_L3:
-          gst_caps_set_simple (new_caps, "profile", G_TYPE_STRING,
-              "advanced-real-time-simple", "level", G_TYPE_STRING, "3", NULL);
-          break;
-        case XVID_PROFILE_ARTS_L4:
-          gst_caps_set_simple (new_caps, "profile", G_TYPE_STRING,
-              "advanced-real-time-simple", "level", G_TYPE_STRING, "4", NULL);
-          break;
-        case XVID_PROFILE_AS_L0:
-          gst_caps_set_simple (new_caps, "profile", G_TYPE_STRING,
-              "advanced-simple", "level", G_TYPE_STRING, "0", NULL);
-          break;
-        case XVID_PROFILE_AS_L1:
-          gst_caps_set_simple (new_caps, "profile", G_TYPE_STRING,
-              "advanced-simple", "level", G_TYPE_STRING, "1", NULL);
-          break;
-        case XVID_PROFILE_AS_L2:
-          gst_caps_set_simple (new_caps, "profile", G_TYPE_STRING,
-              "advanced-simple", "level", G_TYPE_STRING, "2", NULL);
-          break;
-        case XVID_PROFILE_AS_L3:
-          gst_caps_set_simple (new_caps, "profile", G_TYPE_STRING,
-              "advanced-simple", "level", G_TYPE_STRING, "3", NULL);
-          break;
-        case XVID_PROFILE_AS_L4:
-          gst_caps_set_simple (new_caps, "profile", G_TYPE_STRING,
-              "advanced-simple", "level", G_TYPE_STRING, "4", NULL);
-          break;
-        default:
-          g_assert_not_reached ();
-          break;
-      }
-    }
-
     /* src pad should accept anyway */
     ret = gst_pad_set_caps (xvidenc->srcpad, new_caps);
-    gst_caps_unref (new_caps);
+    if (!ret)
+      goto exit;
+    /* will peer accept */
+    peer = gst_pad_get_peer (xvidenc->srcpad);
+    if (peer)
+      ret &= gst_pad_accept_caps (peer, new_caps);
+    gst_object_unref (peer);
 
+  exit:
     if (!ret && xvidenc->handle) {
       xvid_encore (xvidenc->handle, XVID_ENC_DESTROY, NULL, NULL);
       xvidenc->handle = NULL;
     }
+    gst_caps_unref (new_caps);
     return ret;
 
   } else                        /* setup did not work out */
     return FALSE;
 }
 
-static GstCaps *
-gst_xvidenc_getcaps (GstPad * pad)
-{
-  GstXvidEnc *xvidenc;
-  GstPad *peer;
-  GstCaps *caps;
-
-  /* If we already have caps return them */
-  if (GST_PAD_CAPS (pad))
-    return gst_caps_ref (GST_PAD_CAPS (pad));
-
-  xvidenc = GST_XVIDENC (gst_pad_get_parent (pad));
-  if (!xvidenc)
-    return gst_caps_new_empty ();
-
-  peer = gst_pad_get_peer (xvidenc->srcpad);
-  if (peer) {
-    const GstCaps *templcaps;
-    GstCaps *peercaps;
-    guint i, n;
-
-    peercaps = gst_pad_get_caps (peer);
-
-    /* Translate peercaps to YUV */
-    peercaps = gst_caps_make_writable (peercaps);
-    n = gst_caps_get_size (peercaps);
-    for (i = 0; i < n; i++) {
-      GstStructure *s = gst_caps_get_structure (peercaps, i);
-
-      gst_structure_set_name (s, "video/x-raw-yuv");
-      gst_structure_remove_field (s, "mpegversion");
-      gst_structure_remove_field (s, "systemstream");
-    }
-
-    templcaps = gst_pad_get_pad_template_caps (pad);
-
-    caps = gst_caps_intersect (peercaps, templcaps);
-    gst_caps_unref (peercaps);
-    gst_object_unref (peer);
-    peer = NULL;
-  } else {
-    caps = gst_caps_copy (gst_pad_get_pad_template_caps (pad));
-  }
-
-  gst_object_unref (xvidenc);
-
-  return caps;
-}
-
 /* encodes frame according to info in xframe;
    - buf is input buffer, can be NULL if dummy
    - buf is disposed of prior to exit
@@ -1066,10 +806,8 @@ gst_xvidenc_encode (GstXvidEnc * xvidenc, GstBuffer * buf,
     gst_buffer_unref (outbuf);
     return NULL;
   } else if (ret > 0) {         /* make sub-buffer */
-    GstBuffer *sub;
-
     GST_DEBUG_OBJECT (xvidenc, "xvid produced output of size %d", ret);
-    sub = gst_buffer_create_sub (outbuf, 0, ret);
+    GstBuffer *sub = gst_buffer_create_sub (outbuf, 0, ret);
 
     /* parent no longer needed, will go away with child buffer */
     gst_buffer_unref (outbuf);
@@ -1141,8 +879,8 @@ gst_xvidenc_chain (GstPad * pad, GstBuffer * buf)
       xframe.par = XVID_PAR_11_VGA;
     else {
       xframe.par = XVID_PAR_EXT;
-      xframe.par_width = xvidenc->par_width;
-      xframe.par_height = xvidenc->par_height;
+      xframe.par_width = xvidenc->par_height;
+      xframe.par_height = xvidenc->par_width;
     }
 
     /* handle options */
@@ -1226,6 +964,9 @@ gst_xvidenc_chain (GstPad * pad, GstBuffer * buf)
   if (!outbuf)                  /* error or no data yet */
     return GST_FLOW_OK;
 
+  /* proclaim destiny */
+  g_signal_emit (G_OBJECT (xvidenc), gst_xvidenc_signals[FRAME_ENCODED], 0);
+
   /* go out, multiply! */
   return gst_pad_push (xvidenc->srcpad, outbuf);
 }
@@ -1279,6 +1020,7 @@ gst_xvidenc_set_property (GObject * object,
   GstXvidEnc *xvidenc;
   guint offset;
 
+  g_return_if_fail (GST_IS_XVIDENC (object));
   xvidenc = GST_XVIDENC (object);
 
   if (prop_id > xvidenc_prop_count) {
@@ -1289,9 +1031,7 @@ gst_xvidenc_set_property (GObject * object,
   /* our param specs should have such qdata */
   offset =
       GPOINTER_TO_UINT (g_param_spec_get_qdata (pspec, xvidenc_pspec_quark));
-
-  if (offset == 0)
-    return;
+  g_return_if_fail (offset != 0);
 
   switch (G_PARAM_SPEC_VALUE_TYPE (pspec)) {
     case G_TYPE_BOOLEAN:
@@ -1319,9 +1059,10 @@ gst_xvidenc_get_property (GObject * object,
     guint prop_id, GValue * value, GParamSpec * pspec)
 {
   GstXvidEnc *xvidenc;
-  guint offset;
 
+  g_return_if_fail (GST_IS_XVIDENC (object));
   xvidenc = GST_XVIDENC (object);
+  guint offset;
 
   if (prop_id > xvidenc_prop_count) {
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -1331,9 +1072,7 @@ gst_xvidenc_get_property (GObject * object,
   /* our param specs should have such qdata */
   offset =
       GPOINTER_TO_UINT (g_param_spec_get_qdata (pspec, xvidenc_pspec_quark));
-
-  if (offset == 0)
-    return;
+  g_return_if_fail (offset != 0);
 
   switch (G_PARAM_SPEC_VALUE_TYPE (pspec)) {
     case G_TYPE_BOOLEAN:
